// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package dedent

import (
    . "launchpad.net/gocheck"
    "testing"
)

type DedentSuite struct{}

var _ = Suite(&DedentSuite{})

// Dedent() does nothing with the empty string.
func (suite *DedentSuite) TestEmptyString(c *C) {
    input := ""
    expected := input
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() does nothing to a single line without an indent.
func (suite *DedentSuite) TestSingleLine(c *C) {
    input := "This is a single line."
    expected := input
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() removes all leading whitespace from single lines.
func (suite *DedentSuite) TestSingleLineWithIndent(c *C) {
    input := "  This is a single line."
    expected := "This is a single line."
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() does nothing when none of the lines are indented.
func (suite *DedentSuite) TestLines(c *C) {
    input := "One\nTwo\n"
    expected := input
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() does nothing when *any* line is not indented.
func (suite *DedentSuite) TestLinesWithSomeIndents(c *C) {
    input := "One\n  Two\n"
    expected := input
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() removes the common leading indent from each line.
func (suite *DedentSuite) TestLinesWithIndents(c *C) {
    input := " One\n  Two\n"
    expected := "One\n Two\n"
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() ignores all-whitespace lines for the purposes of margin
// calculation. However, the margin *is* trimmed from these lines, if they
// begin with it.
func (suite *DedentSuite) TestLinesWithEmptyLine(c *C) {
    input := "  One\n    \n  Three\n"
    expected := "One\n  \nThree\n"
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() ignores blank lines for the purposes of margin calculation,
// including the first line.
func (suite *DedentSuite) TestLinesWithEmptyFirstLine(c *C) {
    input := "\n  Two\n  Three\n"
    expected := "\nTwo\nThree\n"
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Dedent() treats spaces and tabs as completely different; no number of
// spaces is equivalent to a tab.
func (suite *DedentSuite) TestLinesWithTabsAndSpaces(c *C) {
    input := "\tOne\n        Two\n"
    expected := input
    observed := Dedent(input)
    c.Check(observed, Equals, expected)
}

// Master loader for all tests.
func Test(t *testing.T) {
    TestingT(t)
}
