// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "crypto/rand"
    "crypto/rsa"
    "crypto/x509"
    "crypto/x509/pkix"
    "encoding/pem"
    "fmt"
    . "launchpad.net/gocheck"
    "math/big"
    "os"
    "path"
    "time"
)

type X509SessionSuite struct{}

var _ = Suite(&X509SessionSuite{})

// Create a cert and pem file in a temporary dir in /tmp and return the
// names of the files.  The caller is responsible for cleaning up the files.
func makeX509Certificate() (string, string) {
    // Code is shamelessly stolen from
    // http://golang.org/src/pkg/crypto/tls/generate_cert.go
    priv, err := rsa.GenerateKey(rand.Reader, 1024)
    if err != nil {
        panic(fmt.Errorf("Failed to generate rsa key: %v", err))
    }

    // Create a template for x509.CreateCertificate.
    now := time.Now()
    template := x509.Certificate{
        SerialNumber: new(big.Int).SetInt64(0),
        Subject: pkix.Name{
            CommonName:   "localhost",
            Organization: []string{"Bogocorp"},
        },
        NotBefore:    now.Add(-5 * time.Minute).UTC(),
        NotAfter:     now.AddDate(1, 0, 0).UTC(), // valid for 1 year.
        SubjectKeyId: []byte{1, 2, 3, 4},
        KeyUsage: x509.KeyUsageKeyEncipherment |
            x509.KeyUsageDigitalSignature,
    }

    // Create the certificate itself.
    derBytes, err := x509.CreateCertificate(
        rand.Reader, &template, &template, &priv.PublicKey, priv)
    if err != nil {
        panic(fmt.Errorf("Failed to generate x509 certificate: %v", err))
    }

    // Write the certificate file out.
    dirname := os.TempDir() + "/" + MakeRandomString(10)
    os.Mkdir(dirname, 0700)
    certFile := dirname + "/cert.pem"
    certOut, err := os.Create(certFile)
    if err != nil {
        panic(fmt.Errorf("Failed to create %s: %v", certFile, err))
    }
    pem.Encode(certOut, &pem.Block{Type: "CERTIFICATE", Bytes: derBytes})
    certOut.Close()

    // Write the key file out.
    keyFile := dirname + "/key.pem"
    keyOut, err := os.OpenFile(
        keyFile, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0600)
    if err != nil {
        panic(fmt.Errorf("Failed to create %s: %v", keyFile, err))
    }
    pem.Encode(
        keyOut,
        &pem.Block{
            Type:  "RSA PRIVATE KEY",
            Bytes: x509.MarshalPKCS1PrivateKey(priv)})
    keyOut.Close()

    return certFile, keyFile
}

func (suite *X509SessionSuite) TestNewX509SessionErrorsOnBadCertFile(c *C) {
    creds := Creds{user: "user", key: "key"}
    session, err := NewX509Session(creds, "badcertfile", "badkeyfile")
    c.Check(session, IsNil)
    msg := err.(error).Error()
    c.Check(msg, Matches, "failed to load X509 certificate.*no such file.*")
}

func (suite *X509SessionSuite) TestNewX509SessionCreation(c *C) {
    creds := Creds{user: "user", key: "key"}
    certFile, keyFile := makeX509Certificate()
    session, err := NewX509Session(creds, certFile, keyFile)
    defer func() {
        path := path.Dir(certFile)
        os.RemoveAll(path)
    }()

    c.Assert(err, IsNil)
    c.Assert(session.client, NotNil)
}

func (suite *X509SessionSuite) TestNewX509SessionWithoutCert(c *C) {
    creds := Creds{user: "user", key: "key"}
    sess, err := NewX509SessionWithoutCert(creds)
    c.Assert(err, IsNil)
    c.Assert(sess.client, NotNil)
}
