// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import "encoding/xml"

// It's impossible to have any kind of common method inherited into all the
// various serializable objects because the receiver of the method is the
// wrong class which confuses the xml marshaller.  Hence, this mess.
type AzureObject interface {
    Serialize() (string, error)
}

func toxml(obj AzureObject) (string, error) {
    out, err := xml.MarshalIndent(obj, "", "  ")
    if err != nil {
        return "", err
    }
    return string(out), nil
}

//
// LinuxProvisioningConfiguration bits
//

type LinuxProvisioningConfiguration struct {
    ConfigurationSetType             string `xml:"ConfigurationSetType"` // "LinuxProvisioningConfiguration"
    Hostname                         string `xml:"HostName"`
    Username                         string `xml:"UserName"`
    Password                         string `xml:"UserPassword"`
    DisableSshPasswordAuthentication bool   `xml:"DisableSshPasswordAuthentication"`
}

func (c *LinuxProvisioningConfiguration) Serialize() (string, error) {
    return toxml(c)
}

func NewLinuxProvisioningConfiguration(
    Hostname, Username, Password string,
    DisableSshPasswordAuthentication bool) *LinuxProvisioningConfiguration {
    return &LinuxProvisioningConfiguration{
        ConfigurationSetType: "LinuxProvisioningConfiguration",
        Hostname:             Hostname,
        Username:             Username,
        Password:             Password,
        DisableSshPasswordAuthentication: DisableSshPasswordAuthentication,
    }
}

//
// InputEndpoint bits
//

type InputEndpoint struct {
    LoadBalancedEndpointSetName string `xml:"LoadBalancedEndpointSetName"`
    LocalPort                   int    `xml:"LocalPort"`
    Name                        string `xml:"Name"`
    Port                        int    `xml:"Port"`
    Protocol                    string `xml:"Protocol"` // TCP or UDP
}

func (c *InputEndpoint) Serialize() (string, error) {
    return toxml(c)
}

//
// OSVirtualHardDisk bits
//

type HostCachingType string

const (
    HostCachingRO HostCachingType = "ReadOnly"
    HostCachingRW HostCachingType = "ReadWrite"
)

type OSVirtualHardDisk struct {
    HostCaching     string `xml:"HostCaching"`
    DiskLabel       string `xml:"DiskLabel"`
    DiskName        string `xml:"DiskName"`
    MediaLink       string `xml:"MediaLink"`
    SourceImageName string `xml:"SourceImageName"`
}

func (c *OSVirtualHardDisk) Serialize() (string, error) {
    return toxml(c)
}

func NewOSVirtualHardDisk(HostCaching HostCachingType, DiskLabel string,
    DiskName string, MediaLink string,
    SourceImageName string) *OSVirtualHardDisk {
    return &OSVirtualHardDisk{
        HostCaching:     string(HostCaching),
        DiskLabel:       DiskLabel,
        DiskName:        DiskName,
        MediaLink:       MediaLink,
        SourceImageName: SourceImageName,
    }
}

//
// NetworkConfiguration bits
//

type NetworkConfiguration struct {
    InputEndpoints []InputEndpoint `xml:"InputEndpoints>InputEndpoint"`
}

func (c *NetworkConfiguration) Serialize() (string, error) {
    return toxml(c)
}

type Role struct {
    RoleSize         string                           `xml:"RoleSize"`
    RoleName         string                           `xml:"RoleName"`
    RoleType         string                           `xml:"RoleType"` // Always "PersistentVMRole"
    ConfigurationSet []LinuxProvisioningConfiguration `xml:"ConfigurationSets>ConfigurationSet"`
}

//
// Role bits
///

func (c *Role) Serialize() (string, error) {
    return toxml(c)
}

func NewRole(RoleSize string, RoleName string,
    ConfigurationSet []LinuxProvisioningConfiguration) *Role {
    return &Role{
        RoleSize:         RoleSize,
        RoleName:         RoleName,
        RoleType:         "PersistentVMRole",
        ConfigurationSet: ConfigurationSet,
    }
}

//
// DnsServer bits
//

type DnsServer struct {
    Name    string `xml:"Name"`
    Address string `xml:"Address"`
}

func (c *DnsServer) Serialize() (string, error) {
    return toxml(c)
}

//
// Deployment bits
//

type Deployment struct {
    XMLNS              string      `xml:"xmlns,attr"`
    XMLNS_I            string      `xml:"xmlns:i,attr"`
    Name               string      `xml:"Name"`
    DeploymentSlot     string      `xml:"DeploymentSlot"`
    Label              string      `xml:"Label"`
    RoleList           []Role      `xml:"RoleList>Role"`
    VirtualNetworkName string      `xml:"VirtualNetworkName"`
    Dns                []DnsServer `xml:"Dns>DnsServers>DnsServer"`
}

func (c *Deployment) Serialize() (string, error) {
    return toxml(c)
}

const XMLNS = "http://schemas.microsoft.com/windowsazure"
const XMLNS_I = "http://www.w3.org/2001/XMLSchema-instance"

//
// Storage Services bits
//

type ExtendedProperty struct {
    Name  string `xml:"Name"`
    Value string `xml:"Value"`
}

type StorageService struct {
    // List Storage Accounts.
    // See http://msdn.microsoft.com/en-us/library/windowsazure/ee460787.aspx
    Url                   string             `xml:"Url"`
    ServiceName           string             `xml:"ServiceName"`
    Description           string             `xml:"StorageServiceProperties>Description"`
    AffinityGroup         string             `xml:"StorageServiceProperties>AffinityGroup"`
    Label                 string             `xml:"StorageServiceProperties>Label"` // base64
    Status                string             `xml:"StorageServiceProperties>Status"`
    Endpoints             []string           `xml:"StorageServiceProperties>Endpoints>Endpoint"`
    GeoReplicationEnabled string             `xml:"StorageServiceProperties>GeoReplicationEnabled"`
    GeoPrimaryRegion      string             `xml:"StorageServiceProperties>GeoPrimaryRegion"`
    StatusOfPrimary       string             `xml:"StorageServiceProperties>StatusOfPrimary"`
    LastGeoFailoverTime   string             `xml:"StorageServiceProperties>LastGeoFailoverTime"`
    GeoSecondaryRegion    string             `xml:"StorageServiceProperties>GeoSecondaryRegion"`
    StatusOfSecondary     string             `xml:"StorageServiceProperties>StatusOfSecondary"`
    ExtendedProperties    []ExtendedProperty `xml:"StorageServiceProperties>ExtendedProperties>ExtendedProperty"`

    // TODO: Add accessors for non-string data encoded as strings.
}

type StorageServices struct {
    xmlns           string           `xml:"xmlns,attr"`
    StorageServices []StorageService `xml:"StorageService"`
}

func (s *StorageServices) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}

type CreateStorageServiceInput struct {
    // See http://msdn.microsoft.com/en-us/library/windowsazure/hh264518.aspx
    XMLNS                 string             `xml:"xmlns,attr"`
    ServiceName           string             `xml:"ServiceName"`
    Description           string             `xml:"Description"`
    Label                 string             `xml:"Label"`
    AffinityGroup         string             `xml:"AffinityGroup"`
    Location              string             `xml:"Location"`
    GeoReplicationEnabled string             `xml:"GeoReplicationEnabled"`
    ExtendedProperties    []ExtendedProperty `xml:"ExtendedProperties>ExtendedProperty"`
}

func (c *CreateStorageServiceInput) Serialize() (string, error) {
    return toxml(c)
}

type StorageAccountKeys struct {
    // See http://msdn.microsoft.com/en-us/library/windowsazure/ee460785.aspx
    XMLName   xml.Name `xml:"StorageService"`
    Url       string   `xml:"Url"`
    Primary   string   `xml:"StorageServiceKeys>Primary"`
    Secondary string   `xml:"StorageServiceKeys>Secondary"`
}

func (s *StorageAccountKeys) Deserialize(data []byte) error {
    return xml.Unmarshal(data, s)
}
