// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "fmt"
    . "launchpad.net/gocheck"
    "launchpad.net/gwacl/dedent"
)

type XmlSuite struct{}

var _ = Suite(&XmlSuite{})

//
// Tests for Marshallers
//

func (suite *XmlSuite) TestLinuxProvisioningConfiguration(c *C) {
    config := makeLinuxProvisioningConfiguration()

    xml, err := config.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <LinuxProvisioningConfiguration>
          <ConfigurationSetType>LinuxProvisioningConfiguration</ConfigurationSetType>
          <HostName>%s</HostName>
          <UserName>%s</UserName>
          <UserPassword>%s</UserPassword>
          <DisableSshPasswordAuthentication>%v</DisableSshPasswordAuthentication>
        </LinuxProvisioningConfiguration>`)
    expected := fmt.Sprintf(template, config.Hostname, config.Username,
        config.Password, config.DisableSshPasswordAuthentication)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestInputEndpoint(c *C) {
    endpoint := makeEndpoint(endpointParams{})

    xml, err := endpoint.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <InputEndpoint>
          <LoadBalancedEndpointSetName>%s</LoadBalancedEndpointSetName>
          <LocalPort>%v</LocalPort>
          <Name>%s</Name>
          <Port>%v</Port>
          <Protocol>%s</Protocol>
        </InputEndpoint>`)
    expected := fmt.Sprintf(template, endpoint.LoadBalancedEndpointSetName,
        endpoint.LocalPort, endpoint.Name, endpoint.Port, endpoint.Protocol)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestOSVirtualHardDisk(c *C) {
    disk := makeOSVirtualHardDisk()

    xml, err := disk.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <OSVirtualHardDisk>
          <HostCaching>%s</HostCaching>
          <DiskLabel>%s</DiskLabel>
          <DiskName>%s</DiskName>
          <MediaLink>%s</MediaLink>
          <SourceImageName>%s</SourceImageName>
        </OSVirtualHardDisk>`)
    expected := fmt.Sprintf(template, disk.HostCaching, disk.DiskLabel,
        disk.DiskName, disk.MediaLink, disk.SourceImageName)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestNetworkConfiguration(c *C) {
    endpoint1 := makeEndpoint(endpointParams{})
    endpoint2 := makeEndpoint(endpointParams{})
    endpoints := []InputEndpoint{*endpoint1, *endpoint2}
    config := &NetworkConfiguration{
        InputEndpoints: endpoints}
    xml, err := config.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <NetworkConfiguration>
          <InputEndpoints>
            <InputEndpoint>
              <LoadBalancedEndpointSetName>%s</LoadBalancedEndpointSetName>
              <LocalPort>%v</LocalPort>
              <Name>%s</Name>
              <Port>%v</Port>
              <Protocol>%s</Protocol>
            </InputEndpoint>
            <InputEndpoint>
              <LoadBalancedEndpointSetName>%s</LoadBalancedEndpointSetName>
              <LocalPort>%v</LocalPort>
              <Name>%s</Name>
              <Port>%v</Port>
              <Protocol>%s</Protocol>
            </InputEndpoint>
          </InputEndpoints>
        </NetworkConfiguration>`)
    expected := fmt.Sprintf(template, endpoint1.LoadBalancedEndpointSetName,
        endpoint1.LocalPort, endpoint1.Name, endpoint1.Port,
        endpoint1.Protocol, endpoint2.LoadBalancedEndpointSetName,
        endpoint2.LocalPort, endpoint2.Name, endpoint2.Port,
        endpoint2.Protocol)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestRole(c *C) {
    role := makeRole()
    config := role.ConfigurationSet[0]

    xml, err := role.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <Role>
          <RoleSize>%s</RoleSize>
          <RoleName>%s</RoleName>
          <RoleType>PersistentVMRole</RoleType>
          <ConfigurationSets>
            <ConfigurationSet>
              <ConfigurationSetType>%s</ConfigurationSetType>
              <HostName>%s</HostName>
              <UserName>%s</UserName>
              <UserPassword>%s</UserPassword>
              <DisableSshPasswordAuthentication>%v</DisableSshPasswordAuthentication>
            </ConfigurationSet>
          </ConfigurationSets>
        </Role>`)
    expected := fmt.Sprintf(template, role.RoleSize, role.RoleName,
        config.ConfigurationSetType, config.Hostname, config.Username,
        config.Password, config.DisableSshPasswordAuthentication)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestDeployment(c *C) {
    deployment := makeDeployment()
    dns := deployment.Dns[0]
    role := deployment.RoleList[0]
    config := role.ConfigurationSet[0]

    xml, err := deployment.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <Deployment xmlns="http://schemas.microsoft.com/windowsazure" xmlns:i="http://www.w3.org/2001/XMLSchema-instance">
          <Name>%s</Name>
          <DeploymentSlot>%s</DeploymentSlot>
          <Label>%s</Label>
          <RoleList>
            <Role>
              <RoleSize>%s</RoleSize>
              <RoleName>%s</RoleName>
              <RoleType>PersistentVMRole</RoleType>
              <ConfigurationSets>
                <ConfigurationSet>
                  <ConfigurationSetType>%s</ConfigurationSetType>
                  <HostName>%s</HostName>
                  <UserName>%s</UserName>
                  <UserPassword>%s</UserPassword>
                  <DisableSshPasswordAuthentication>%v</DisableSshPasswordAuthentication>
                </ConfigurationSet>
              </ConfigurationSets>
            </Role>
          </RoleList>
          <VirtualNetworkName>%s</VirtualNetworkName>
          <Dns>
            <DnsServers>
              <DnsServer>
                <Name>%s</Name>
                <Address>%s</Address>
              </DnsServer>
            </DnsServers>
          </Dns>
        </Deployment>`)
    expected := fmt.Sprintf(template, deployment.Name,
        deployment.DeploymentSlot, deployment.Label, role.RoleSize,
        role.RoleName, config.ConfigurationSetType, config.Hostname,
        config.Username, config.Password,
        config.DisableSshPasswordAuthentication,
        deployment.VirtualNetworkName, dns.Name, dns.Address)
    c.Check(xml, Equals, expected)
}

func (suite *XmlSuite) TestCreateStorageServiceInput(c *C) {
    s := makeCreateStorageServiceInput()
    ext_property := s.ExtendedProperties[0]
    xml, err := s.Serialize()
    c.Assert(err, IsNil)
    template := dedent.Dedent(`
        <CreateStorageServiceInput xmlns="http://schemas.microsoft.com/windowsazure">
          <ServiceName>%s</ServiceName>
          <Description>%s</Description>
          <Label>%s</Label>
          <AffinityGroup>%s</AffinityGroup>
          <Location>%s</Location>
          <GeoReplicationEnabled>%s</GeoReplicationEnabled>
          <ExtendedProperties>
            <ExtendedProperty>
              <Name>%s</Name>
              <Value>%s</Value>
            </ExtendedProperty>
          </ExtendedProperties>
        </CreateStorageServiceInput>`)
    expected := fmt.Sprintf(template, s.ServiceName, s.Description, s.Label,
        s.AffinityGroup, s.Location, s.GeoReplicationEnabled, ext_property.Name,
        ext_property.Value)
    c.Assert(xml, Equals, expected)
}

//
// Tests for Unmarshallers
//

func (suite *XmlSuite) TestStorageServicesUnmarshal(c *C) {
    input_template := `
        <?xml version="1.0" encoding="utf-8"?>
        <StorageServices xmlns="http://schemas.microsoft.com/windowsazure">
          <StorageService>
            <Url>%s</Url>
            <ServiceName>%s</ServiceName>
            <StorageServiceProperties>
              <Description>%s</Description>
              <AffinityGroup>%s</AffinityGroup>
              <Label>%s</Label>
              <Status>%s</Status>
              <Endpoints>
                <Endpoint>%s</Endpoint>
                <Endpoint>%s</Endpoint>
                <Endpoint>%s</Endpoint>
              </Endpoints>
              <GeoReplicationEnabled>%s</GeoReplicationEnabled>
              <GeoPrimaryRegion>%s</GeoPrimaryRegion>
              <StatusOfPrimary>%s</StatusOfPrimary>
              <LastGeoFailoverTime>%s</LastGeoFailoverTime>
              <GeoSecondaryRegion>%s</GeoSecondaryRegion>
              <StatusOfSecondary>%s</StatusOfSecondary>
              <ExtendedProperties>
                <ExtendedProperty>
                  <Name>%s</Name>
                  <Value>%s</Value>
                </ExtendedProperty>
                <ExtendedProperty>
                  <Name>%s</Name>
                  <Value>%s</Value>
                </ExtendedProperty>
              </ExtendedProperties>
            </StorageServiceProperties>
          </StorageService>
        </StorageServices>`
    url := MakeRandomString(10)
    servicename := MakeRandomString(10)
    desc := MakeRandomString(10)
    affinity := MakeRandomString(10)
    label := MakeRandomString(10)
    status := MakeRandomString(10)
    blob_endpoint := MakeRandomString(10)
    queue_endpoint := MakeRandomString(10)
    table_endpoint := MakeRandomString(10)
    geo_repl := BoolToString(MakeRandomBool())
    geo_region := MakeRandomString(10)
    status_primary := MakeRandomString(10)
    failover_time := MakeRandomString(10)
    geo_sec_region := MakeRandomString(10)
    status_sec := MakeRandomString(10)
    p1_name := MakeRandomString(10)
    p1_val := MakeRandomString(10)
    p2_name := MakeRandomString(10)
    p2_val := MakeRandomString(10)

    input := fmt.Sprintf(input_template, url, servicename, desc, affinity,
        label, status, blob_endpoint, queue_endpoint, table_endpoint, geo_repl,
        geo_region, status_primary, failover_time, geo_sec_region, status_sec,
        p1_name, p1_val, p2_name, p2_val)
    data := []byte(input)

    services := &StorageServices{}
    err := services.Deserialize(data)
    c.Assert(err, IsNil)

    c.Check(len(services.StorageServices), Equals, 1)
    s := services.StorageServices[0]

    // Oh jeez, here we go....
    c.Check(s.Url, Equals, url)
    c.Check(s.ServiceName, Equals, servicename)
    c.Check(s.Description, Equals, desc)
    c.Check(s.AffinityGroup, Equals, affinity)
    c.Check(s.Label, Equals, label)
    c.Check(s.Status, Equals, status)
    c.Check(s.GeoReplicationEnabled, Equals, geo_repl)
    c.Check(s.GeoPrimaryRegion, Equals, geo_region)
    c.Check(s.StatusOfPrimary, Equals, status_primary)
    c.Check(s.LastGeoFailoverTime, Equals, failover_time)
    c.Check(s.GeoSecondaryRegion, Equals, geo_sec_region)
    c.Check(s.StatusOfSecondary, Equals, status_sec)

    endpoints := s.Endpoints
    c.Check(len(endpoints), Equals, 3)
    c.Check(endpoints[0], Equals, blob_endpoint)
    c.Check(endpoints[1], Equals, queue_endpoint)
    c.Check(endpoints[2], Equals, table_endpoint)

    properties := s.ExtendedProperties
    c.Check(properties[0].Name, Equals, p1_name)
    c.Check(properties[0].Value, Equals, p1_val)
    c.Check(properties[1].Name, Equals, p2_name)
    c.Check(properties[1].Value, Equals, p2_val)
}

func (suite *XmlSuite) TestStorageAccountKeysUnmarshal(c *C) {
    template := `
        <?xml version="1.0" encoding="utf-8"?>
          <StorageService xmlns="http://schemas.microsoft.com/windowsazure">
            <Url>%s</Url>
            <StorageServiceKeys>
              <Primary>%s</Primary>
              <Secondary>%s</Secondary>
            </StorageServiceKeys>
          </StorageService>`
    url := MakeRandomString(10)
    key1 := MakeRandomString(10)
    key2 := MakeRandomString(10)
    input := fmt.Sprintf(template, url, key1, key2)
    data := []byte(input)

    keys := &StorageAccountKeys{}
    err := keys.Deserialize(data)
    c.Assert(err, IsNil)
    c.Check(keys.Url, Equals, url)
    c.Check(keys.Primary, Equals, key1)
    c.Check(keys.Secondary, Equals, key2)
}

// Tests for object factory functions.

func (suite *XmlSuite) TestNewRole(c *C) {
    rolesize := MakeRandomString(10)
    rolename := MakeRandomString(10)
    config := makeLinuxProvisioningConfiguration()
    configset := []LinuxProvisioningConfiguration{*config}

    role := NewRole(rolesize, rolename, configset)
    c.Check(role.RoleSize, Equals, rolesize)
    c.Check(role.RoleName, Equals, rolename)
    c.Check(role.ConfigurationSet, DeepEquals, configset)
    c.Check(role.RoleType, Equals, "PersistentVMRole")
}

func (suite *XmlSuite) TestNewLinuxProvisioningConfiguration(c *C) {
    hostname := MakeRandomString(10)
    username := MakeRandomString(10)
    password := MakeRandomString(10)
    disablessh := MakeRandomBool()

    config := NewLinuxProvisioningConfiguration(
        hostname, username, password, disablessh)
    c.Check(config.Hostname, Equals, hostname)
    c.Check(config.Username, Equals, username)
    c.Check(config.Password, Equals, password)
    c.Check(config.DisableSshPasswordAuthentication, Equals, disablessh)
    c.Check(config.ConfigurationSetType, Equals, "LinuxProvisioningConfiguration")
}

func (suite *XmlSuite) TestNewOSVirtualHardDisk(c *C) {
    var hostcaching HostCachingType = "ReadWrite"
    disklabel := MakeRandomString(10)
    diskname := MakeRandomString(10)
    MediaLink := MakeRandomString(10)
    SourceImageName := MakeRandomString(10)

    disk := NewOSVirtualHardDisk(
        hostcaching, disklabel, diskname, MediaLink, SourceImageName)
    c.Check(disk.HostCaching, Equals, string(hostcaching))
    c.Check(disk.DiskLabel, Equals, disklabel)
    c.Check(disk.DiskName, Equals, diskname)
    c.Check(disk.MediaLink, Equals, MediaLink)
    c.Check(disk.SourceImageName, Equals, SourceImageName)
}
